@@ -1,17 +1,21 @@
+#include <complex>
+
 #include <cosma/communicator.hpp>
 #include <cosma/one_sided_communicator.hpp>
 #include <cosma/two_sided_communicator.hpp>
 
-#include <complex>
+#if defined(COSMA_HAVE_GPU) && defined(COSMA_WITH_NCCL)
+#include <cosma/gpu/nccl_utils.hpp>
+#endif
 
 namespace cosma {
 bool communicator::use_busy_waiting = true;
 
-communicator::communicator(const Strategy *strategy, 
+communicator::communicator(const Strategy strategy, 
                            MPI_Comm comm)
     : strategy_(strategy) {
 
-    use_busy_waiting = strategy_->use_busy_waiting;
+    use_busy_waiting = strategy_.use_busy_waiting;
 
     MPI_Comm_rank(comm, &rank_);
     // rank_ = reordered_rank(rank_);
@@ -19,14 +23,14 @@ communicator::communicator(const Strategy *strategy,
     // check if the reordered rank belongs 
     // to this communicator
     assert(rank_ < comm_size_);
-    using_reduced_comm_ = comm_size_ != strategy->P;
-    is_idle_ = rank_ >= strategy->P;
+    using_reduced_comm_ = comm_size_ != strategy.P;
+    is_idle_ = rank_ >= strategy.P;
 
     if (using_reduced_comm_) {
         MPI_Group group;
         MPI_Comm_group(comm, &group);
         std::vector<int> exclude_ranks;
-        for (int i = strategy->P; i < comm_size_; ++i) {
+        for (int i = strategy.P; i < comm_size_; ++i) {
             // exclude_ranks.push_back(reordered_rank(i));
             exclude_ranks.push_back(i);
         }
@@ -42,25 +46,30 @@ communicator::communicator(const Strategy *strategy,
         MPI_Group_free(&group);
         MPI_Group_free(&reduced_group);
     } else {
-        full_comm_ = comm;
+        // this communicator has to be duplicated as it's being cached.
+        // The user might deallocate the comm as it's allocated outside of COSMA
+        // for this reason, we have to ensure that we duplicate the comm
+        // before it's cached in the COSMA context
+        MPI_Comm_dup(comm, &full_comm_);
+        // full_comm_ = comm;
     }
 
     if (is_idle_) {
         return;
     }
 
-    if (strategy_->topology) {
+    if (strategy_.topology) {
         add_topology();
     }
 
     create_communicators(full_comm_);
     // split_communicators(full_comm_);
 
-    step_to_comm_index_ = std::vector<int>(strategy_->n_steps());
+    step_to_comm_index_ = std::vector<int>(strategy_.n_steps());
     int idx = 0;
-    for (int i = 0; i < strategy_->n_steps(); ++i) {
+    for (int i = 0; i < strategy_.n_steps(); ++i) {
         step_to_comm_index_[i] = idx;
-        if (strategy_->parallel_step(i))
+        if (strategy_.parallel_step(i))
             idx++;
     }
 }
@@ -79,19 +88,19 @@ bool communicator::is_idle() { return is_idle_; }
 // weight of the edge is given by the amount of communicated data
 void communicator::get_topology_edges(std::vector<int> &dest,
                                       std::vector<int> &weight) {
-    int m = strategy_->m;
-    int n = strategy_->n;
-    int k = strategy_->k;
-    Interval P(0, strategy_->P - 1);
-    int n_steps = strategy_->n_steps();
+    int m = strategy_.m;
+    int n = strategy_.n;
+    int k = strategy_.k;
+    Interval P(0, strategy_.P - 1);
+    int n_steps = strategy_.n_steps();
 
     for (int step = 0; step < n_steps; ++step) {
-        m /= strategy_->divisor_m(step);
-        n /= strategy_->divisor_n(step);
-        k /= strategy_->divisor_k(step);
+        m /= strategy_.divisor_m(step);
+        n /= strategy_.divisor_n(step);
+        k /= strategy_.divisor_k(step);
 
-        if (strategy_->parallel_step(step)) {
-            int div = strategy_->divisor(step);
+        if (strategy_.parallel_step(step)) {
+            int div = strategy_.divisor(step);
             int partition_idx = P.subinterval_index(div, rank_);
             Interval newP = P.subinterval(div, partition_idx);
             int group, offset;
@@ -105,9 +114,9 @@ void communicator::get_topology_edges(std::vector<int> &dest,
                 dest.push_back(neighbor);
 
                 int communication_size = 0;
-                if (strategy_->split_n(step))
+                if (strategy_.split_n(step))
                     communication_size = m * k / newP.length();
-                else if (strategy_->split_m(step))
+                else if (strategy_.split_m(step))
                     communication_size = k * n / newP.length();
                 else
                     communication_size = m * n / newP.length();
@@ -165,9 +174,22 @@ MPI_Comm communicator::active_comm(int step) {
     return comm_ring_[comm_index];
 }
 
+#ifdef COSMA_WITH_NCCL
+ncclComm_t communicator::active_nccl_comm(int step) {
+    int comm_index = step_to_comm_index_[step];
+    return nccl_comm_ring_[comm_index];
+}
+#endif
+
 int communicator::comm_size() { return comm_size_; }
 
-void communicator::free_comm(MPI_Comm &comm) { MPI_Comm_free(&comm); }
+void communicator::free_comm(MPI_Comm &comm) { 
+    int mpi_finalized;
+    MPI_Finalized(&mpi_finalized);
+    if (!mpi_finalized) {
+        MPI_Comm_free(&comm); 
+    }
+}
 
 void communicator::free_group(MPI_Group &group) { MPI_Group_free(&group); }
 
@@ -212,12 +234,12 @@ int communicator::rank_outside_ring(Interval &P, int div, int off, int i) {
 
 void communicator::split_communicators(MPI_Comm comm) {
     // MPI_Comm_group(comm, &comm_group);
-    Interval P(0, strategy_->P - 1);
+    Interval P(0, strategy_.P - 1);
     // iterate through all steps and for each parallel
     // step, create a suitable subcommunicator
-    for (int step = 0; step < strategy_->n_steps(); ++step) {
-        if (strategy_->parallel_step(step)) {
-            int div = strategy_->divisor(step);
+    for (int step = 0; step < strategy_.n_steps(); ++step) {
+        if (strategy_.parallel_step(step)) {
+            int div = strategy_.divisor(step);
             int partition_idx = P.subinterval_index(div, rank_);
             Interval newP = P.subinterval(div, partition_idx);
             int group, offset;
@@ -225,8 +247,15 @@ void communicator::split_communicators(MPI_Comm comm) {
             MPI_Comm comm_ring, comm_subproblem;
             MPI_Comm_split(comm, group, offset, &comm_subproblem);
             MPI_Comm_split(comm, offset, group, &comm_ring);
+
             comm_ring_.push_back(comm_ring);
             comm_subproblem_.push_back(comm_subproblem);
+
+#ifdef COSMA_WITH_NCCL
+            nccl_comm_ring_.push_back(gpu::mpi_to_nccl_comm(comm_ring_.back()));
+            nccl_comm_subproblem_.push_back(gpu::mpi_to_nccl_comm(comm_subproblem_.back()));
+#endif
+
             comm = comm_subproblem;
             P = newP;
         }
@@ -252,13 +281,13 @@ MPI_Comm create_comm(MPI_Comm& comm, std::vector<int>& ranks) {
 
 void communicator::create_communicators(MPI_Comm comm) {
     // MPI_Comm_group(comm, &comm_group);
-    Interval P(0, strategy_->P - 1);
+    Interval P(0, strategy_.P - 1);
 
     // iterate through all steps and for each parallel
     // step, create a suitable subcommunicator
-    for (int step = 0; step < strategy_->n_steps(); ++step) {
-        if (strategy_->parallel_step(step)) {
-            int div = strategy_->divisor(step);
+    for (int step = 0; step < strategy_.n_steps(); ++step) {
+        if (strategy_.parallel_step(step)) {
+            int div = strategy_.divisor(step);
             int partition_idx = P.subinterval_index(div, rank_);
             Interval newP = P.subinterval(div, partition_idx);
             int group, offset;
@@ -267,6 +296,11 @@ void communicator::create_communicators(MPI_Comm comm) {
             comm_ring_.emplace_back(create_comm_ring(comm, P, offset, div));
             comm_subproblem_.emplace_back(create_comm_subproblem(comm, P, newP));
 
+#ifdef COSMA_WITH_NCCL
+            nccl_comm_ring_.emplace_back(gpu::mpi_to_nccl_comm(comm_ring_.back()));
+            nccl_comm_subproblem_.emplace_back(gpu::mpi_to_nccl_comm(comm_subproblem_.back()));
+#endif
+
             comm = comm_subproblem_.back();
             P = newP;
         }
@@ -311,13 +345,20 @@ MPI_Comm communicator::create_comm_subproblem(MPI_Comm comm,
 void communicator::free_comms() {
     for (int i = comm_subproblem_.size() - 1; i >= 0; --i) {
         free_comm(comm_subproblem_[i]);
+#ifdef COSMA_WITH_NCCL
+        gpu::free_nccl_comm(nccl_comm_subproblem_[i]);
+#endif
     }
     for (int i = comm_ring_.size() - 1; i >= 0; --i) {
         free_comm(comm_ring_[i]);
+#ifdef COSMA_WITH_NCCL
+        gpu::free_nccl_comm(nccl_comm_ring_[i]);
+#endif
     }
-    if (using_reduced_comm_) {
-        free_comm(full_comm_);
-    }
+    // if (using_reduced_comm_) {
+    free_comm(full_comm_);
+    full_comm_ = MPI_COMM_NULL;
+    //}
 }
 
 template <typename Scalar>
@@ -332,7 +373,7 @@ void communicator::copy(Interval &P,
     MPI_Comm comm = active_comm(step);
     two_sided_communicator::copy(comm,
                                  rank(),
-                                 strategy_->divisor(step),
+                                 strategy_.divisor(step),
                                  P,
                                  in,
                                  out,
@@ -358,7 +399,7 @@ void communicator::reduce(Interval &P,
     MPI_Comm comm = active_comm(step);
     two_sided_communicator::reduce(comm,
                                    rank(),
-                                   strategy_->divisor(step),
+                                   strategy_.divisor(step),
                                    P,
                                    in,  // LC
                                    out, // C
@@ -400,6 +441,10 @@ void communicator::overlap_comm_and_comp(cosma_context<Scalar> *ctx,
                                                   beta);
 }
 
+const Strategy communicator::get_strategy() {
+    return strategy_;
+}
+
 // Explicit instantiations for `copy`
 //
 template void
