@@ -292,7 +292,7 @@ void pxgemm(const char transa,
 #endif
 
     // first, we don't want to alloate the space, just to precompute
-    // the required memory size, so we active dry_run, which precomputes
+    // the required memory size, so we activate dry_run, which precomputes
     // everything but doesn't allocate anything yet
     bool dont_allocate = true;
     CosmaMatrix<T> A(std::move(mapper_a), rank_permutation[rank], dont_allocate);
@@ -300,9 +300,22 @@ void pxgemm(const char transa,
     CosmaMatrix<T> C(std::move(mapper_c), rank_permutation[rank], dont_allocate);
 
     // avoid resizing the buffer by reserving immediately the total required memory
-    get_context_instance<T>()->get_memory_pool().reserve(A.total_required_memory()
-                                                       + B.total_required_memory()
-                                                       + C.total_required_memory());
+    // collect sizes of all buffers that are going to be allocated for each matrix
+    auto A_buffers = A.required_memory();
+    auto B_buffers = B.required_memory();
+    auto C_buffers = C.required_memory();
+
+    std::vector<std::size_t> buffer_sizes;
+    int n_buffers = A_buffers.size() + B_buffers.size() + C_buffers.size();
+    if (n_buffers > 0) {
+        buffer_sizes.reserve(n_buffers);
+        std::copy(A_buffers.begin(), A_buffers.end(), std::back_inserter(buffer_sizes));
+        std::copy(B_buffers.begin(), B_buffers.end(), std::back_inserter(buffer_sizes));
+        std::copy(C_buffers.begin(), C_buffers.end(), std::back_inserter(buffer_sizes));
+
+        // allocate all buffers in the memory pool
+        get_context_instance<T>()->get_memory_pool().reserve(buffer_sizes);
+    }
 
     // turn off dryrun mode, allocate memory for all matrices
     A.allocate();
@@ -352,25 +365,21 @@ void pxgemm(const char transa,
 
 #ifdef DEBUG
     if (rank == 0) {
-        auto reordered_vol = costa::communication_volume(scalapack_layout_a.grid, cosma_layout_a.grid);
-        reordered_vol += costa::communication_volume(scalapack_layout_b.grid, cosma_layout_b.grid);
+        auto reordered_vol = costa::communication_volume(scalapack_layout_a.grid, cosma_layout_a.grid, trans_a);
+        reordered_vol += costa::communication_volume(scalapack_layout_b.grid, cosma_layout_b.grid, trans_b);
         if (std::abs(beta) > 0) {
-            reordered_vol += costa::communication_volume(scalapack_layout_c.grid, cosma_layout_c.grid);
+            reordered_vol += costa::communication_volume(scalapack_layout_c.grid, cosma_layout_c.grid, 'N');
         }
-        reordered_vol += costa::communication_volume(cosma_layout_c.grid, scalapack_layout_c.grid);
+        reordered_vol += costa::communication_volume(cosma_layout_c.grid, scalapack_layout_c.grid, 'N');
 
         // std::cout << "Detailed comm volume: " << comm_vol << std::endl;
         // std::cout << "Detailed comm volume reordered: " << reordered_vol << std::endl;
 
-        auto comm_vol_total = comm_vol.total_volume();
         auto reordered_vol_total = reordered_vol.total_volume();
-        std::cout << "Initial comm volume = " << comm_vol_total << std::endl;
-        std::cout << "Reduced comm volume = " << reordered_vol_total << std::endl;
-        auto diff = (long long) comm_vol_total - (long long) reordered_vol_total;
-        std::cout << "Comm volume reduction [%] = " << 100.0 * diff / comm_vol_total << std::endl;
-
+        std::cout << "Comm volume [num_of_elements]= " << reordered_vol_total << std::endl;
     }
 #endif
+
     PE(transform_reordering_comm);
     if (reordered) {
         MPI_Comm_free(&reordered_comm);
