@@ -60,7 +60,7 @@ TEST_P(PdgemmTestWithParams, pdgemm) {
             std::cout << state << std::endl;
         }
 
-        cosma::Strategy::min_dim_size = 32;
+        // cosma::Strategy::min_dim_size = 32;
         bool correct = test_pxgemm<double>(state, comm);
         EXPECT_TRUE(correct);
 
@@ -602,6 +602,305 @@ INSTANTIATE_TEST_CASE_P(
             0, 0, // matrix A
             0, 0, // matrix B
             0, 0  // matrix C
+        },
+
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            1824, 128, // matrix A
+            1824, 128, // matrix B
+            128, 128, // matrix C
+
+            // block sizes
+            32, 32, // matrix A
+            32, 32, // matrix B
+            32, 32, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            128, 128, 1824,
+
+            // transpose flags
+            'T', 'N',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            928, 928, 64,
+
+            // proc grid
+            2, 4, 'R',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            43417, 217, // matrix A
+            43417, 217, // matrix B
+            217, 217, // matrix C
+
+            // block sizes
+            169, 108, // matrix A
+            169, 108, // matrix B
+            108, 108, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            217, 217, 43417,
+
+            // transpose flags
+            'T', 'N',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            54272, 54272, 1088,
+
+            // proc grid
+            8, 1, 'R',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            43176, 217, // matrix A
+            43176, 217, // matrix B
+            2176, 217, // matrix C
+
+            // block sizes
+            1696, 108, // matrix A
+            1696, 108, // matrix B
+            1088, 108, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            217, 217, 43176,
+
+            // transpose flags
+            'T', 'N',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            54272, 54272, 1088,
+
+            // proc grid
+            8, 1, 'R',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+	// CP2K runs from H2O-sos-mp2-lr
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            23, 23, // matrix A
+            23, 23, // matrix B
+            23, 23, // matrix C
+
+            // block sizes
+            12, 12, // matrix A
+            12, 12, // matrix B
+            12, 12, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            23, 23, 4,
+
+            // transpose flags
+            'N', 'T',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            12, 12, 12,
+
+            // proc grid
+            2, 1, 'C',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            23, 23, // matrix A
+            23, 23, // matrix B
+            23, 23, // matrix C
+
+            // block sizes
+            12, 12, // matrix A
+            12, 12, // matrix B
+            12, 12, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            23, 23, 4,
+
+            // transpose flags
+            'N', 'T',
+
+            // scaling flags
+            1.0, -1.0,
+
+            // leading dims
+            12, 12, 12,
+
+            // proc grid
+            2, 1, 'C',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            83, 83, // matrix A
+            83, 83, // matrix B
+            83, 83, // matrix C
+
+            // block sizes
+            32, 32, // matrix A
+            32, 32, // matrix B
+            32, 32, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            83, 83, 83,
+
+            // transpose flags
+            'N', 'T',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            83, 83, 83,
+
+            // proc grid
+            1, 1, 'C',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            83, 83, // matrix A
+            83, 77, // matrix B
+            83, 77, // matrix C
+
+            // block sizes
+            32, 32, // matrix A
+            32, 32, // matrix B
+            32, 32, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            83, 77, 83,
+
+            // transpose flags
+            'T', 'N',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            83, 83, 83,
+
+            // proc grid
+            1, 1, 'C',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
+        },
+
+        cosma::pxgemm_params<double>{
+            // matrix dimensions
+            83, 83, // matrix A
+            83, 77, // matrix B
+            83, 77, // matrix C
+
+            // block sizes
+            32, 32, // matrix A
+            32, 32, // matrix B
+            32, 32, // matrix C
+
+            // submatrices ij
+            1, 1, // matrix A
+            1, 1, // matrix B
+            1, 1, // matrix C
+
+            // problem size
+            83, 77, 83,
+
+            // transpose flags
+            'T', 'N',
+
+            // scaling flags
+            1.0, 0.0,
+
+            // leading dims
+            83, 83, 83,
+
+            // proc grid
+            1, 1, 'R',
+
+            // proc srcs
+            0, 0, // matrix A
+            0, 0, // matrix B
+            0, 0  // matrix C
         }
-    ));
+    )
+);
 
